@@ -300,6 +300,8 @@
 #define DEFAULT_HIGH_BIT_ESCAPE 2
 #define DEFAULT_HOLD_SLIDER 100
 #define DEFAULT_INDENT 0
+#define DEFAULT_INPUT_INDICATOR_LEFT "+ "
+#define DEFAULT_INPUT_INDICATOR_RIGHT " +"
 #define DEFAULT_INPUT_PROMPT "> "
 #define DEFAULT_INSERT_MODE 1
 #define DEFAULT_INVERSE_VIDEO 1
