@@ -45,7 +45,9 @@
 #include <unistd.h>
 #include <signal.h>
 #include <limits.h>
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <errno.h>
 #include <sys/stat.h>
 
@@ -230,6 +232,8 @@ int	unblock_signal (int);
 sigfunc *my_signal (int, sigfunc *);
 #define SIGNAL_HANDLER(x) \
 	RETSIGTYPE x (int unused)
+sigfunc *init_signals (void);
+extern	volatile int    signals_caught[NSIG];
 
 #define BUILT_IN_COMMAND(x) \
 	void x (const char *command, char *args, const char *subargs)
@@ -253,6 +257,22 @@ typedef char Filename[MAXPATHLEN + 1];
 #define memmove(x, y, z) bcopy(y, x, z)
 #endif
 
+/*
+ * Interix's getpgrp() does not take an argument, but the configure script
+ * detects it wrongly.
+ */
+#ifdef __INTERIX
+# define GETPGRP_VOID
+#endif
+
+/*
+ * Interix provides intptr_t, but the shipped gcc 3.3 provides a broken
+ * stddef.h that hides it from us.
+ */
+#ifdef __INTERIX
+typedef int intptr_t;
+#endif
+
 /*
  * Figure out our intmax_t
  *
@@ -261,6 +281,10 @@ typedef char Filename[MAXPATHLEN + 1];
  * provided in compat.c.  If you have (quad_t), you had better have strtoq(),
  * and you have to have strtol() in any circumstance.
  */
+#if defined(HAVE_INTMAX_NATIVE) && !defined(HAVE_STRTOIMAX)
+# undef HAVE_INTMAX_NATIVE
+#endif
+
 #ifndef HAVE_INTMAX_NATIVE
 # ifdef HAVE_INTMAX_LONG_LONG
 #  define intmax_t long long
@@ -415,4 +439,12 @@ typedef struct hostent		Hostent;
 typedef struct timeval		Timeval;
 typedef struct stat		Stat;
 
+/*
+ * Interix's getpgrp() does not take an argument, but the configure script
+ * detects it wrongly.
+ */
+#ifdef __INTERIX
+# define GETPGRP_VOID
+#endif
+
 #endif /* __irc_std_h */
