@@ -2,7 +2,7 @@
  * server.h: header for server.c 
  *
  * Copyright 1990 Michael Sandrof
- * Copyright 1997 EPIC Software Labs
+ * Copyright 1997, 2007 EPIC Software Labs
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  */
 
@@ -52,15 +52,6 @@ typedef	struct
 {
 	ServerInfo *info;		/* Canonical information */
 
-#if 0
-	char	*name;			/* the name of the server */
-	int	port;			/* port number on that server */
-	char	*password;		/* password for that server */
-	char	*group;			/* Server group it belongs to */
-	int	try_ssl;		/* SSL requested on next connection. */
-	int	protocol;		/* AF_UNSPEC, AF_INET, or AF_INET6 */
-#endif
-
 	AI 	*addrs;			/* Returned by getaddrinfo */
 const	AI	*next_addr;		/* The next one to try upon failure */
 	int	addr_counter;		/* How far we're into "addrs" */
@@ -109,6 +100,8 @@ const	AI	*next_addr;		/* The next one to try upon failure */
 	char	*quit_message;		/* Where we stash a quit message */
 	A005	a005;			/* 005 settings kept kere. */
 	int	stricmp_table;		/* Which case insensitive map to use */
+	int	autoclose;		/* Whether the server is closed when
+					   there are no windows on it */
 
 	int	funny_min;		/* Funny stuff */
 	int	funny_max;
@@ -117,6 +110,9 @@ const	AI	*next_addr;		/* The next one to try upon failure */
 
 	int	ssl_enabled;		/* Current SSL status. */
 
+	char *	realname;		/* The actual realname. */
+	char *	default_realname;	/* The default realname. */
+
         int             doing_privmsg;
         int             doing_notice;
         int             doing_ctcp;
@@ -132,6 +128,7 @@ const	AI	*next_addr;		/* The next one to try upon failure */
         char *          recv_nick;
         char *          sent_nick;
         char *          sent_body;
+
 }	Server;
 extern	Server	**server_list;
 #endif	/* NEED_SERVER_LIST */
@@ -338,6 +335,8 @@ const	char*	get_server_005			(int, const char *);
 	int	get_server_userhost_max		(int);
 	void	set_server_status		(int, int);
 	int	get_server_status		(int);
+	void	set_server_autoclose		(int, int);
+	int	get_server_autoclose		(int);
 
         void    set_server_invite_channel       (int, const char *);
 const char *    get_server_invite_channel       (int);
@@ -361,6 +360,10 @@ const char *	get_server_cookie         	(int);
 const char *	get_server_last_notify_nick    	(int);
 	void	set_server_unique_id		(int, const char *);
 const char *	get_server_unique_id    	(int);
+	void	set_server_realname		(int, const char *);
+const char *	get_server_realname		(int);
+	void	set_server_default_realname	(int, const char *);
+const char *	get_server_default_realname	(int);
 
 	void	set_server_funny_min         	(int, int);
 	int	get_server_funny_min         	(int);
