@@ -60,6 +60,8 @@ int	ALLOW_C1_CHARS_VAR,
 	HIGH_BIT_ESCAPE_VAR,
 	HOLD_SLIDER_VAR,
 	INDENT_VAR,
+	INPUT_INDICATOR_LEFT_VAR,
+        INPUT_INDICATOR_RIGHT_VAR,
 	INPUT_PROMPT_VAR,
 	INSERT_MODE_VAR,
 	KEY_INTERVAL_VAR,
