@@ -1,4 +1,4 @@
-/* $EPIC: commands.c,v 1.171 2007/09/07 18:07:29 jnelson Exp $ */
+/* $EPIC: commands.c,v 1.182 2008/05/15 18:04:24 alex Exp $ */
 /*
  * commands.c -- Stuff needed to execute commands in ircII.
  *		 Includes the bulk of the built in commands for ircII.
@@ -37,6 +37,7 @@
 #define __need_putchar_x__
 #define __need_term_flush__
 #include "irc.h"
+#define __need_ArgList_t__
 #include "alias.h"
 #include "alist.h"
 #include "sedcrypt.h"
@@ -47,7 +48,7 @@
 #include "files.h"
 #include "hook.h"
 #include "server.h"
-#include "if.h"
+#include "ifcmd.h"
 #include "ignore.h"
 #include "input.h"
 #include "ircaux.h"
@@ -63,7 +64,7 @@
 #include "screen.h"
 #include "status.h"
 #include "stack.h"
-#include "term.h"
+#include "termx.h"
 #include "timer.h"
 #include "vars.h"
 #include "window.h"
@@ -72,6 +73,7 @@
 #include "words.h"
 #include "reg.h"
 #include "extlang.h"
+#include "elf.h"
 
 /* used with input_move_cursor */
 #define RIGHT 1
@@ -972,7 +974,18 @@ BUILT_IN_COMMAND(xechocmd)
 			    if (!(w = get_window_by_refnum(
 					get_channel_winref(flag_arg, 
 							from_server))))
+			{
+			    /* 
+			     * This is a special favor to Blackjac for
+			     * backwards compatability with epic4 where
+			     * /xecho -w $winchan() would output to the 
+			     * current window when $winchan() returned -1.
+			     */
+			    if (!my_stricmp(flag_arg, "-1"))
+				w = current_window;
+			    else
 				return;	/* No such window */
+			}
 
 			to_window_refnum = w->refnum;
 			break;
@@ -988,7 +1001,6 @@ BUILT_IN_COMMAND(xechocmd)
 			else
 				all_windows = 1;
 
-			next_arg(args, &args);
 			break;
 		}
 
@@ -1001,6 +1013,8 @@ BUILT_IN_COMMAND(xechocmd)
 
 		case 'R':   /* RAW OUTPUT TO TERMINAL */
 		{
+			Window *wx;
+
 			next_arg(args, &args);
 			/*
 			 * Nuke reminded me of this.  Just because to_window
@@ -1009,8 +1023,9 @@ BUILT_IN_COMMAND(xechocmd)
 			 * So we have to make sure that output_screen is
 			 * to_window->screen.
 			 */
-			if (to_window_refnum != -1)
-				output_screen = get_window_by_refnum(to_window_refnum)->screen;
+			if (to_window_refnum != -1 && 
+			    ((wx = get_window_by_refnum(to_window_refnum))))
+				output_screen = wx->screen;
 			else
 				output_screen = current_window->screen;
 			tputs_x(args);
@@ -1041,7 +1056,7 @@ BUILT_IN_COMMAND(xechocmd)
 			break;
 		}
 
-		case 'F': /* DO not notify for hidden windwos (%F) */
+		case 'F': /* DO not notify for hidden windows (%F) */
 		{
 			next_arg(args, &args);
 			do_window_notifies = 0;
@@ -1555,9 +1570,10 @@ BUILT_IN_COMMAND(subpackagecmd)
 	malloc_strcat(&load_level[load_depth].package, args);
 }
 
-static void	loader_which (FILE *fp, const char *filename, const char *args, struct load_info *);
-static void	loader_std (FILE *fp, const char *filename, const char *args, struct load_info *);
-static void	loader_pf  (FILE *fp, const char *filename, const char *args, struct load_info *);
+static void	loader_which (struct epic_loadfile *elf, const char *filename, const char *args, struct load_info *);
+static void	loader_std (struct epic_loadfile *elf, const char *filename, const char *args, struct load_info *);
+static void	loader_pf  (struct epic_loadfile *elf, const char *filename, const char *args, struct load_info *);
+
 /*
  * load: the /LOAD command.  Reads the named file, parsing each line as
  * though it were typed in (passes each line to parse_statement). 
@@ -1570,9 +1586,16 @@ BUILT_IN_COMMAND(load)
 	char *	use_path;
 	char *	expanded;
 	FILE *	fp;
+        struct epic_loadfile * elf;
 	int	display;
 	int	do_one_more = 0;
-	void	(*loader) (FILE *, const char *, const char *, struct load_info *);
+	void	(*loader) (struct epic_loadfile *, const char *, const char *, struct load_info *);
+
+        int             idx;
+        int             err;
+        char            errstr[1024];
+        char            buf[8192];
+
 
 	if (++load_depth == MAX_LOAD_DEPTH)
 	{
@@ -1650,24 +1673,25 @@ BUILT_IN_COMMAND(load)
 	     * uzfopen() also frees 'expanded' for us on error.
 	     */
 	    expanded = malloc_strdup(filename);
-	    
-	    if (!(fp = uzfopen(&expanded, use_path, 1, &load_level[load_depth].sb)))
-		continue;
 
-	    load_level[load_depth].filename = expanded;
+            if (!(elf = uzfopen(&expanded, use_path, 1, &load_level[load_depth].sb)))
+                continue;
+
+            load_level[load_depth].filename = expanded;
 	    load_level[load_depth].line = 1;
 	    if (load_depth > 0 && load_level[load_depth - 1].package)
 	        malloc_strcpy(&load_level[load_depth].package,
 				load_level[load_depth-1].package);
 
 	    will_catch_return_exceptions++;
-	    loader(fp, expanded, sargs, &load_level[load_depth]);
+	    loader(elf, expanded, sargs, &load_level[load_depth]);
 	    will_catch_return_exceptions--;
 	    return_exception = 0;
 
 	    new_free(&load_level[load_depth].filename);
 	    new_free(&load_level[load_depth].package);
-	    fclose(fp);
+            epic_fclose(elf);
+            new_free(&elf);
 	}
 
 	/*
@@ -1682,14 +1706,14 @@ BUILT_IN_COMMAND(load)
 }
 
 /* The "WHICH" loader */
-static void	loader_which (FILE *fp, const char *filename, const char *subargs, struct load_info *loadinfo)
+static void	loader_which (struct epic_loadfile *elf, const char *filename, const char *subargs, struct load_info *loadinfo)
 {
 	loadinfo->loader = "which";
 	yell("%s", filename);
 }
 
 /* The "Standard" (legacy) loader */
-static void	loader_std (FILE *fp, const char *filename, const char *subargs, struct load_info *loadinfo)
+static void	loader_std (struct epic_loadfile *elf, const char *filename, const char *subargs, struct load_info *loadinfo)
 {
 	int	in_comment, comment_line, no_semicolon;
 	int	paste_level, paste_line;
@@ -1697,7 +1721,7 @@ static void	loader_std (FILE *fp, const char *filename, const char *subargs, str
 #define MAX_LINE_SIZE BIG_BUFFER_SIZE * 5
 	char	buffer[MAX_LINE_SIZE * 2 + 1];
 
-	loadinfo->loader = "std";
+        loadinfo->loader = "std";
 
 	in_comment = 0;
 	comment_line = -1;
@@ -1712,15 +1736,17 @@ static void	loader_std (FILE *fp, const char *filename, const char *subargs, str
 	    int     len;
 	    char    *ptr;
 
-	    if (!fgets(buffer, MAX_LINE_SIZE, fp))
-		break;
+            if (!epic_fgets(buffer, MAX_LINE_SIZE, elf))
+                    break;
 
-	    if (loadinfo->line == 1 && loadinfo->sb.st_mode & 0111 &&
+            if (loadinfo->line == 1 && loadinfo->sb.st_mode & 0111 &&
 	    	(buffer[0] != '#' || buffer[1] != '!'))
 	    {
-	    	yell("Cannot open %s -- executable file", loadinfo->filename);
-		return;
+	    	yell("Caution -- %s is marked as an executable; "
+		     "loading binaries results in undefined behavior.", 
+		     loadinfo->filename);
 	    }
+
 	    for (start = buffer; my_isspace(*start); start++)
 		;
 
@@ -1754,7 +1780,7 @@ static void	loader_std (FILE *fp, const char *filename, const char *subargs, str
 			(len >= 2 && start[len-2] == '\\') &&
 			(len < 3 || start[len-3] != '\\') && 
 		        (len < MAX_LINE_SIZE) && 
-			(fgets(&(start[len-2]), MAX_LINE_SIZE - len, fp)))
+                        (epic_fgets(&(start[len-2]), MAX_LINE_SIZE - len, elf)))
 	    {
 		len = strlen(start);
 		loadinfo->line++;
@@ -2016,7 +2042,7 @@ static void	loader_std (FILE *fp, const char *filename, const char *subargs, str
  * and is "signficantly" faster than the standard loader.  This can be a 
  * big win for large scripts.
  */
-static void	loader_pf (FILE *fp, const char *filename, const char *subargs, struct load_info *loadinfo)
+static void	loader_pf (struct epic_loadfile *elf, const char *filename, const char *subargs, struct load_info *loadinfo)
 {
 	char *	buffer;
 	int	bufsize, pos;
@@ -2031,7 +2057,7 @@ static void	loader_pf (FILE *fp, const char *filename, const char *subargs, stru
 	comment = 0;
 	shebang = 0;
 
-	this_char = fgetc(fp);
+	this_char = epic_fgetc(elf);
 
 	/* 
 	 * If the file is +x, and starts with #!, it's ok.
@@ -2040,9 +2066,9 @@ static void	loader_pf (FILE *fp, const char *filename, const char *subargs, stru
 	if (loadinfo->sb.st_mode & 0111)
 	{
 	    /* Special code to support #! scripts. */
-	    if (this_char == '#' && !feof(fp))
+	    if (this_char == '#' && !epic_feof(elf))
 	    {
-		this_char = fgetc(fp);
+		this_char = epic_fgetc(elf);
 		if (this_char == '!')
 		   shebang = 1;
 	    }
@@ -2058,7 +2084,7 @@ static void	loader_pf (FILE *fp, const char *filename, const char *subargs, stru
 	    comment = 1;
 	}
 
-	while (!feof(fp))
+	while (!epic_feof(elf))
 	{
 	    do
 	    {
@@ -2100,7 +2126,7 @@ static void	loader_pf (FILE *fp, const char *filename, const char *subargs, stru
 		new_realloc((void **)&buffer, bufsize);
 	    }
 
-	    this_char = fgetc(fp);
+	    this_char = epic_fgetc(elf);
 	}
 
 	buffer[pos] = 0;
@@ -2213,7 +2239,6 @@ BUILT_IN_COMMAND(quotecmd)
 {
 	int	refnum = from_server;
 	int	urlencoded = 0;
-	size_t	length;
 
 	if (*command == 'X')
 	{
@@ -2261,9 +2286,20 @@ BUILT_IN_COMMAND(quotecmd)
 
 	if (urlencoded)
 	{
-		length = strlen(args);
-		urldecode(args, &length);
-		send_to_aserver_raw(refnum, length, args);
+		size_t  length, 
+			retsize;
+		char    *ret;
+		int     transform, 
+			numargs;
+
+		transform = lookup_transform("URL", &numargs);
+		retsize = strlen(args) + 2;
+		ret = new_malloc(retsize);
+		length = transform_string(transform, 0, NULL, 
+					args, strlen(args), 
+					ret, retsize);
+		send_to_aserver_raw(refnum, length, ret);
+		new_free(&ret);
 	}
 	else if (args && *args)
 		send_to_aserver(refnum, "%s", args);
@@ -3301,7 +3337,14 @@ void 	new_send_text (int server, const char *orig_target_list, const char *text,
  */
 static void	eval_inputlist (char *args, char *line)
 {
-	runcmds(args, line);
+        char *tmp;
+	if (args[0]=='(') {
+                tmp=next_expr(&args, '(');
+                runcmds_with_arglist(args, tmp, line);
+	} else {
+                /* traditional behavior */
+		runcmds(args, line);
+	}
 }
 
 BUILT_IN_COMMAND(e_call)
@@ -3445,7 +3488,18 @@ void	runcmds (const char *what, const char *subargs)
 	parse_block(what, subargs, 0);
 }
 
-/* 
+void    runcmds_with_arglist (const char *what, char *args, char *subargs)
+{
+	ArgList *arglist = NULL;
+	if (!subargs)
+		subargs = LOCAL_COPY(empty_string);
+	arglist = parse_arglist(args);
+	parse_line_alias_special(NULL, what, subargs, arglist, 0);
+
+	destroy_arglist(&arglist);
+}
+
+/*
  * parse_block: execute a block of ircII statements (in a C string)
  *
  * Arguments:
@@ -3761,7 +3815,9 @@ BUILT_IN_COMMAND(allocdumpcmd)
 
 BUILT_IN_COMMAND(botmodecmd)
 {
-#ifndef NO_BOTS
+#if defined(NO_BOTS) || defined(NO_JOB_CONTROL)
+	yell("This client was configured not to allow bots.  Bummer.");
+#else
 	if (dumb_mode) {
 		use_input = 0;
 		background = 1;
@@ -3773,8 +3829,6 @@ BUILT_IN_COMMAND(botmodecmd)
 	} else {
 		say("Bot mode can only be entered from Dumb mode.");
 	}
-#else
-	yell("This client was configured not to allow bots.  Bummer.");
 #endif
 }
 
