@@ -1,4 +1,4 @@
-/* $EPIC: hook.c,v 1.77 2007/09/07 18:07:29 jnelson Exp $ */
+/* $EPIC: hook.c,v 1.81 2008/04/22 23:30:22 howl Exp $ */
 /*
  * hook.c: Does those naughty hook functions. 
  *
@@ -42,7 +42,7 @@
 #include "window.h"
 #include "output.h"
 #include "commands.h"
-#include "if.h"
+#include "ifcmd.h"
 #include "stack.h"
 #include "reg.h"
 #include "functions.h"
@@ -214,6 +214,7 @@ Hookables hook_function_templates[] =
 	{ "SERVER_NOTICE",	NULL,	1,	0,	0,	NULL, 0 },
 	{ "SERVER_STATUS",	NULL,	3,	0,	0,	NULL, 0 },
 	{ "SET",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "SIGNAL",		NULL,	1,	0,	0,	NULL, 0 },
 	{ "SIGNOFF",		NULL,	1,	0,	0,	NULL, 0 },
 	{ "SILENCE",		NULL,	2,	0,	0,	NULL, 0 },
 	{ "SSL_SERVER_CERT",	NULL,	3,	0,	0,	NULL, 0 },
@@ -232,6 +233,7 @@ Hookables hook_function_templates[] =
 	{ "WINDOW_CREATE",	NULL,	1, 	0,	0,	NULL, 0 },
 	{ "WINDOW_BEFOREKILL",	NULL,	1,	0,	0,	NULL, 0 },
 	{ "WINDOW_KILL",	NULL,	2,	0,	0,	NULL, 0 },
+	{ "WINDOW_NOTIFIED",	NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
 	{ "WINDOW_SERVER",	NULL,	3,	0,	0,	NULL, 0 },
 	{ "YELL",		NULL,	1,	0,	0,	NULL, 0 },
 };
@@ -620,10 +622,10 @@ void	unload_on_hooks (char *filename)
 static void 	show_hook (Hook *list, const char *name)
 {
 	char *arglist;
-	arglist = print_arglist(list->arglist);
-	if (arglist)
+
+	if ((arglist = print_arglist(list->arglist)))
 	{
-		say ("[%s] On %s from %c%s%c (%s) do %s [%s] <%d/#%d>",
+		say ("[%s] On %s from %c%s%c (%s) do %s [%s%s] <%d/#%d>",
 			empty(list->filename) ? "*" : list->filename,
 			name,
 			(list->flexible ? '\'' : '"'),
@@ -632,17 +634,19 @@ static void 	show_hook (Hook *list, const char *name)
 			arglist,
 			(list->not ? "nothing" : list->stuff),
 			noise_info[list->noisy]->name,
+			list->skip ? "/DISABLED" : "",
 			list->sernum, list->userial);
 		new_free(&arglist);
 	}
 	else
-		say("[%s] On %s from %c%s%c do %s [%s] <%d/#%d>",
+		say("[%s] On %s from %c%s%c do %s [%s%s] <%d/#%d>",
 		    empty(list->filename) ? "*" : list->filename,
 	    	name,
   	  	(list->flexible ? '\'' : '"'), list->nick, 
  		   (list->flexible ? '\'' : '"'), 
     		(list->not ? "nothing" : list->stuff),
 	 	   noise_info[list->noisy]->name,
+		list->skip ? "/DISABLED" : "",
 	    	list->sernum, list->userial);
 }
 
