@@ -1,4 +1,4 @@
-/* $EPIC: irc.c,v 1.1181 2007/09/18 02:34:26 jnelson Exp $ */
+/* $EPIC: irc.c,v 1.1225 2008/06/27 01:42:13 jnelson Exp $ */
 /*
  * ircII: a new irc client.  I like it.  I hope you will too!
  *
@@ -6,7 +6,7 @@
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
  * Copyright  1994 Jake Khuon.
- * Copyright  1993, 2007 EPIC Software Labs.
+ * Copyright  1993, 2008 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,19 +40,19 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC5-0.3.5";
-const char useful_info[] = "epic5 0 3 5";
+const char irc_version[] = "EPIC5-0.3.9";
+const char useful_info[] = "epic5 0 3 9";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20070917";
+const char internal_version[] = "20080626";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 1509;
+const unsigned long	commit_id = 1554;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -60,7 +60,7 @@ const unsigned long	commit_id = 1509;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Alacrity";
+const char ridiculous_version_name[] = "Contumacy";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -71,7 +71,7 @@ const char ridiculous_version_name[] = "Alacrity";
 #include "input.h"
 #include "alias.h"
 #include "output.h"
-#include "term.h"
+#include "termx.h"
 #include "exec.h"
 #include "screen.h"
 #include "log.h"
@@ -376,6 +376,8 @@ The program will now terminate.						\n", irc_version, commit_id, internal_versi
 
 		fflush(stdout);
 		panic_dump_call_stack();
+
+		while ((x_debug & DEBUG_CRASH) && !sleep(1)) {};
 	}
 
         if (x_debug & DEBUG_CRASH)
@@ -425,7 +427,7 @@ static SIGNAL_HANDLER(cntl_c)
 	if (cntl_c_hit++ >= 4)
 		irc_exit(1, "User pressed ^C five times.");
 	else if (cntl_c_hit > 1)
-		kill(getpid(), SIGALRM);
+		kill(getpid(), SIGUSR2);
 }
 
 static SIGNAL_HANDLER(nothing)
@@ -631,8 +633,11 @@ static	void	parse_args (int argc, char **argv)
 		
 		for (argn = 1; argn < argc; argn++)
 		{
-		    int arglen = strlen(argv[argn]);
-		    char buffer[arglen +1];
+		    int arglen;
+		    char *buffer;
+
+		    arglen =  strlen(argv[argn]);
+		    buffer = alloca(arglen + 1);
 		    argpos = -1;
 		    do
 		    {
@@ -838,6 +843,23 @@ static	void	parse_args (int argc, char **argv)
 	return;
 }
 
+/* fire scripted signal events -pegasus */
+static void	do_signals(void)
+{
+	int sig_no;
+
+	signals_caught[0] = 0;
+	for (sig_no = 0; sig_no < NSIG; sig_no++)
+	{
+		while (signals_caught[sig_no])
+		{
+			do_hook(SIGNAL_LIST, "%d %d", sig_no, 
+					signals_caught[sig_no]);
+			signals_caught[sig_no]--;
+		}
+	}
+}
+
 /* 
  * io() is a ONE TIME THROUGH loop!  It simply does ONE check on the
  * file descriptors, and if there is nothing waiting, it will time
@@ -934,6 +956,10 @@ static	int		level = 0,
 	 * Various things that need to be done synchronously...
 	 */
 
+	/* deal with caught signals - pegasus */
+	if (signals_caught[0] != 0)
+		do_signals();
+
 	/* Account for dead child processes */
 	get_child_exit(-1);
 
@@ -1139,6 +1165,7 @@ void    load_ircrc (void)
 /*************************************************************************/
 int 	main (int argc, char *argv[])
 {
+	setvbuf(stdout, NULL, _IOLBF, 1024);
 #ifdef SOCKS
 	SOCKSinit(argv[0]);
 #endif
@@ -1168,6 +1195,7 @@ int 	main (int argc, char *argv[])
 				irc_version, commit_id, internal_version);
 	fprintf(stderr, "%s\n", compile_info);
 
+#ifndef NO_JOB_CONTROL
 	/* If we're a bot, do the bot thing. */
 	if (!use_input)
 	{
@@ -1188,6 +1216,7 @@ int 	main (int argc, char *argv[])
 		}
 	}
 	else
+#endif
 	{
 		fprintf(stderr, "Process [%d]", getpid());
 		if (isatty(0))
@@ -1197,6 +1226,8 @@ int 	main (int argc, char *argv[])
 		fprintf(stderr, "\n");
 	}
 
+	init_signals();
+
 	/* these should be taken by both dumb and smart displays */
 	my_signal(SIGSEGV, coredump);
 	my_signal(SIGBUS, coredump);
