@@ -1,4 +1,4 @@
-/* $EPIC: screen.c,v 1.128 2007/07/20 22:29:33 jnelson Exp $ */
+/* $EPIC: screen.c,v 1.138 2008/05/15 17:57:54 alex Exp $ */
 /*
  * screen.c
  *
@@ -50,7 +50,7 @@
 #include "vars.h"
 #include "server.h"
 #include "list.h"
-#include "term.h"
+#include "termx.h"
 #include "names.h"
 #include "ircaux.h"
 #include "input.h"
@@ -127,6 +127,7 @@ static void 	add_to_window	(Window *, const unsigned char *);
 static	int	ok_to_output	(Window *);
 static ssize_t read_esc_seq     (const unsigned char *, void *, int *);
 static ssize_t read_color_seq   (const unsigned char *, void *d, int);
+static	void translate_user_input (char byte);
 
 /*
  * "Attributes" were an invention for epic5, and the general idea was
@@ -1694,17 +1695,18 @@ const	unsigned char	*cont_ptr;
 				int	continued_count = 0;
 
 				/* Because Blackjac asked me to */
-				if (indent > max_cols / 3)
+				/* The test against 0 is for lines w/o spaces */
+				if (indent > max_cols / 3 || indent <= 0)
 					indent = max_cols / 3;
 
-				if (do_indent && 
-					((int)strlen(cont_ptr) < indent))
+				if (do_indent)
 				{
-					size_t size = indent + 10;;
+				    unsigned char *fixedstr;
 
-					cont = alloca(size);    /* sb pana */
-					snprintf(cont, size, 
-						"%-*s", indent, cont_ptr);
+				    fixedstr = prepare_display2(cont_ptr, 
+							indent, 0, ' ', 1);
+				    cont = LOCAL_COPY(fixedstr);
+				    new_free(&fixedstr);
 				}
 
 				/*
@@ -1831,6 +1833,119 @@ const	unsigned char	*cont_ptr;
 	return output;
 }
 
+/*
+ * An evil bastard child hack that pulls together the important parts of
+ * prepare_display(), fix_string_width(), and output_with_count().
+ *
+ * This function is evil because it was spawned by copying the above 
+ * functions and they were not refactored to make use of this function,
+ * which they would do if I were not doing this in a rush.
+ *
+ * If you change the above three functions, you would do well to make sure to 
+ * adjust this function, for if you do not, then HIGGLEDY PIGGLEDY WILL ENSUE.
+ *
+ * XXX - This is a bletcherous inelegant hack and i hate it.
+ */
+unsigned char *prepare_display2 (const unsigned char *orig_str, int max_cols, int allow_truncate, char fillchar, int denormalize)
+{
+	int 	pos = 0,            /* Current position in "buffer" */
+		col = 0,            /* Current column in display    */
+		line = 0,           /* Current pos in "output"      */
+		newline = 0;        /* Number of newlines           */
+	unsigned char 	*str = NULL;
+	unsigned char	*retval = NULL;
+	size_t		clue = 0;
+const 	unsigned char	*ptr;
+	unsigned char 	buffer[BIG_BUFFER_SIZE + 1];
+	Attribute	a;
+	unsigned char *	real_retval;
+
+        /* Reset all attributes to zero */
+        a.bold = a.underline = a.reverse = a.blink = a.altchar = 0;
+        a.color_fg = a.color_bg = a.fg_color = a.bg_color = 0;
+
+        str = new_normalize_string(orig_str, 3, NORMALIZE);
+	buffer[0] = 0;
+
+	/*
+	 * Start walking through the entire string.
+	 */
+	for (ptr = str; *ptr && (pos < BIG_BUFFER_SIZE - 8); ptr++)
+	{
+		switch (*ptr)
+		{
+			case '\007':      /* bell */
+				buffer[pos++] = *ptr;
+				break;
+
+			case '\n':      /* Forced newline */
+				newline = 1;
+				break; /* case '\n' */
+
+                        /* Attribute changes -- copy them unmodified. */
+                        case '\006':
+                        {
+                                if (read_attributes(ptr, &a) == 0)
+                                {
+                                        buffer[pos++] = *ptr++;
+                                        buffer[pos++] = *ptr++;
+                                        buffer[pos++] = *ptr++;
+                                        buffer[pos++] = *ptr++;
+                                        buffer[pos++] = *ptr;
+                                }
+                                else
+                                        abort();
+
+                                continue;          /* Skip the column check */
+                        }
+
+			default:
+			{
+				buffer[pos++] = *ptr;
+				col++;
+				break;
+			}
+		} /* End of switch (*ptr) */
+
+		/*
+		 * Must check for cols >= maxcols+1 becuase we can have a 
+		 * character on the extreme screen edge, and we would still 
+		 * want to treat this exactly as 1 line, and col has already 
+		 * been incremented.
+		 */
+		if ((allow_truncate && col > max_cols) || newline)
+			break;
+	}
+	buffer[pos] = 0;
+
+	if (*buffer)
+		malloc_strcpy_c((char **)&retval, buffer, &clue);
+
+	/*
+	 * If we get here, either we have slurped up 'max_cols' cols, or
+	 * we hit a newline, or the string was too short.
+	 */
+	if (col < max_cols && fillchar != '\0')
+	{
+		char fillstr[2];
+		fillstr[0] = fillchar;
+		fillstr[1] = 0;
+
+		/* XXX One col per byte assumption! */
+		while (col++ < max_cols)
+			malloc_strcat_c((char **)&retval, fillstr, &clue);
+	}
+
+	if (denormalize)
+		real_retval = denormalize_string(retval);
+	else
+		real_retval = retval, retval = NULL;
+
+	new_free(&retval);
+	new_free(&str);
+	return real_retval;
+}
+
 /*
  * rite: This is the primary display wrapper to the 'output_with_count'
  * function.  This function is called whenever a line of text is to be
@@ -2230,7 +2345,7 @@ static void 	add_to_window (Window *window, const unsigned char *str)
 	{
 	    const char *type = NULL;
 
-	    /* /WINDOW BEEP_ALWAYS added for archon.  */
+            /* /WINDOW BEEP_ALWAYS added for archon.  */
 	    if (window->beep_always && strchr(str, '\007'))
 	    {
 		type = "Beep";
@@ -2240,6 +2355,7 @@ static void 	add_to_window (Window *window, const unsigned char *str)
 			mask_isset(&window->notify_mask, who_level))
 	    {
 		window->notified = 1;
+	    	do_hook(WINDOW_NOTIFIED_LIST, "%u %s", window->refnum, level_to_str(who_level));
 		if (window->notify_when_hidden)
 			type = "Activity";
 		update_all_status();
@@ -2469,7 +2585,7 @@ void 	repaint_window_body (Window *window)
 		int	cols;
 		int	numls = 1;
 		unsigned char **lines;
-		unsigned char *n;
+		unsigned char *n, *widthstr;
 		const unsigned char *str;
 
 		if (!(str = window->topline[count]))
@@ -2480,11 +2596,19 @@ void 	repaint_window_body (Window *window)
 		term_clear_to_eol();
 		cols = window->columns - 1;
 
-		n = new_normalize_string(str, 0, display_line_mangler);
-		lines = prepare_display(window->refnum, n, cols, &numls, PREPARE_NOWRAP);
+		widthstr = prepare_display2(str, cols, 1, ' ', 0);
+		output_with_count(widthstr, 1, foreground);
+		new_free(&widthstr);
+
+/*
+		n = new_normalize_string(widthstr, 0, display_line_mangler);
+		lines = prepare_display(window->refnum, n, cols, 
+					&numls, PREPARE_NOWRAP);
 		if (*lines)
 			output_with_count(*lines, 1, foreground);
 		new_free(&n);
+*/
+
 		cursor_in_display(window);
 	   }
 
@@ -2585,7 +2709,11 @@ void	create_new_screen (void)
 	new_s->input_visible = 0;
 	new_s->input_start_zone = 0;
 	new_s->input_prompt = malloc_strdup(empty_string);
-	new_s->input_prompt_len = 0;
+        new_s->input_prompt_len = 0;
+        new_s->ind_left = malloc_strdup(empty_string);
+        new_s->ind_left_len = 0;
+        new_s->ind_right = malloc_strdup(empty_string);
+        new_s->ind_right_len = 0;
 	new_s->input_line = 23;
 
 	last_input_screen = new_s;
@@ -2597,12 +2725,15 @@ void	create_new_screen (void)
 }
 
 
-#ifdef WINDOW_CREATE
 #define ST_NOTHING      -1
 #define ST_SCREEN       0
 #define ST_XTERM        1
 Window	*create_additional_screen (void)
 {
+#ifdef NO_JOB_CONTROL
+	yell("Your system doesn't support job control, sorry.");
+	return NULL;
+#else
         Window  	*win;
         Screen  	*oldscreen, *new_s;
         char    	*displayvar,
@@ -2850,6 +2981,7 @@ Window	*create_additional_screen (void)
 	    }
 	}
 	return NULL;
+#endif
 }
 
 /* Old screens never die. They just fade away. */
@@ -2908,7 +3040,6 @@ void 	kill_screen (Screen *screen)
 	make_window_current(NULL);
 	say("The screen is now dead.");
 }
-#endif /* WINDOW_CREATE */
 
 
 /* * * * * * * * * * * * * USER INPUT HANDLER * * * * * * * * * * * */
@@ -2916,6 +3047,11 @@ static void 	do_screens (int fd)
 {
 	Screen *screen;
 	char 	buffer[IO_BUFFER_SIZE + 1];
+	int	saved_from_server;
+	int	n, i;
+	int	proto;
+
+	saved_from_server = from_server;
 
 	if (use_input)
 	for (screen = screen_list; screen; screen = screen->next)
@@ -2923,140 +3059,142 @@ static void 	do_screens (int fd)
 		if (!screen->alive)
 			continue;
 
-#ifdef WINDOW_CREATE
-		/* wserv control */
+		/*
+		 * Handle input from a WSERV screen's control channel
+		 */
 		if (screen->control != -1 && screen->control == fd)
 		{
-			if (dgets(screen->control, buffer, IO_BUFFER_SIZE, 1) < 0)
-			{
-				kill_screen(screen);
-				yell("Error from remote screen.");
-				continue;
-			}
+		    if (dgets(screen->control, buffer, IO_BUFFER_SIZE, 1) < 0)
+		    {
+			kill_screen(screen);
+			yell("Error from remote screen.");
+			continue;
+		    }
 
-			if (!strncmp(buffer, "tty=", 4))
-				malloc_strcpy(&screen->tty_name, buffer + 4);
-			else if (!strncmp(buffer, "geom=", 5))
+		    if (!strncmp(buffer, "tty=", 4))
+			malloc_strcpy(&screen->tty_name, buffer + 4);
+		    else if (!strncmp(buffer, "geom=", 5))
+		    {
+			char *ptr;
+			if ((ptr = strchr(buffer, ' ')))
+				*ptr++ = 0;
+			screen->li = atoi(buffer + 5);
+			screen->co = atoi(ptr);
+			recalculate_windows(screen);
+		    }
+		    else if (!strncmp(buffer, "version=", 8))
+		    {
+			int     version;
+			version = atoi(buffer + 8);
+			if (version != CURRENT_WSERV_VERSION)
 			{
-				char *ptr;
-				if ((ptr = strchr(buffer, ' ')))
-					*ptr++ = 0;
-				screen->li = atoi(buffer + 5);
-				screen->co = atoi(ptr);
-				recalculate_windows(screen);
-			}
-			else if (!strncmp(buffer, "version=", 8))
-			{
-				int     version;
-				version = atoi(buffer + 8);
-				if (version != CURRENT_WSERV_VERSION)
-				{
-				    yell("WSERV version %d is incompatable with this binary",
-						version);
-				    kill_screen(screen);
-				}
-				screen->wserv_version = version;
+			    yell("WSERV version %d is incompatable "
+					"with this binary", version);
+			    kill_screen(screen);
 			}
+			screen->wserv_version = version;
+		    }
 		}
-#endif
 
-		if (screen->fdin == fd)
+
+		/*
+		 * Handle user input from any screen's fdin channel
+		 */
+		if (screen->fdin != fd)
+			continue;
+
+		/*
+		 * If normal data comes in, and the wserv didn't tell me
+		 * what its version is, then throw it out of the system.
+		 */
+		if (screen != main_screen && screen->wserv_version == 0)
 		{
-			int	server;
-
-#ifdef WINDOW_CREATE
-			if (screen != main_screen && screen->wserv_version == 0)
-			{
-				kill_screen(screen);
-				yell("The WSERV used to create this new screen is too old.");
-				return;
-			}
-#endif
-
-			/*
-			 * This section of code handles all in put from 
-			 * the terminal(s) connected to ircII.  Perhaps the 
-			 * idle time *shouldn't* be reset unless its not a 
-			 * screen-fd that was closed..
-			 */
-			get_time(&idle_time);
-			if (cpu_saver)
-				reset_system_timers();
-
-			server = from_server;
-			last_input_screen = screen;
-			output_screen = screen;
-			make_window_current(screen->current_window);
-			/*
-			 * In a multi-screen environment, it's possible for
-			 * the user to "switch" between windows connected to
-			 * the same server on multiple screens; this would
-			 * be the only place we would know about that.  So 
-			 * every time the user presses a key we have to set 
-			 * the screen's current window to be that window's
-			 * server's current window.  Right.
-			 * XXX Why do I know I'm going to regret this?
-			 */
-			current_window->priority = current_window_priority++;
-			from_server = current_window->server;
-
-			if (dumb_mode)
-			{
-				if (dgets(screen->fdin, buffer, IO_BUFFER_SIZE, 1) < 0)
-				{
-					say("IRCII exiting on EOF from stdin");
-					irc_exit(1, "EPIC - EOF from stdin");
-				}
-
-				if (strlen(buffer))
-					buffer[strlen(buffer) - 1] = 0;
-				parse_statement(buffer, 1, NULL);
-			}
-			else
-			{
-				int	n, i;
-
-				/*
-				 * Read in from stdin.
-				 */
-				n = dgets(screen->fdin, buffer,
-						BIG_BUFFER_SIZE, -1);
-				if (n > 0)
-				{
-					int proto = get_server_protocol_state(from_server);
-					set_server_protocol_state(from_server, 0);
-
-					for (i = 0; i < n; i++)
-						edit_char(buffer[i]);
-					set_server_protocol_state(from_server, proto);
-				}
-
-#ifdef WINDOW_CREATE
-				/* 
-				 * If this is a /window create screen, then 
-				 * an EOF or an error means the screen has
-				 * gone away.  We don't need it, so swap out
-				 * all of its windows and kill it off.
-				 */
-				else if (screen != main_screen)
-					kill_screen(screen);
-#endif
-
-				/*
-				 * If we get an EOF or error for the primary 
-				 * screen, then we've lost access to the pty
-				 * (probably because the user has been logged
-				 * out).  There is nothing more we can do but
-				 * throw up our hands and quit.
-				 */
-				else
-					irc_exit(1, "Hey!  Where'd my controlling terminal go?");
-			}
-			from_server = server;
+			kill_screen(screen);
+			yell("The WSERV used to create this new screen "
+				"is too old.");
+			return;		/* Bail out entirely */
 		}
-	} 
+
+		/* Reset the idleness data for the user */
+		get_time(&idle_time);
+		if (cpu_saver)
+			reset_system_timers();
+
+		/* 
+		 * Set the global processing context for the client to the 
+		 * processing context of the screen's current input window.
+		 */
+		last_input_screen = screen;
+		output_screen = screen;
+		make_window_current(screen->current_window);
+		from_server = current_window->server;
+
+		/*
+		 * PRIVMSG/NOTICE restrictions are suspended
+		 * when you're typing something.
+		 */
+		proto = get_server_protocol_state(from_server);
+		set_server_protocol_state(from_server, 0);
+
+		/*
+		 * We create a 'stack' of current windows so whenever a 
+		 * window is killed, we know which window was the current
+		 * window immediately prior to it.
+		 */
+		current_window->priority = current_window_priority++;
+
+		/* Dumb mode only accepts complete lines from the user */
+		if (dumb_mode)
+		{
+			if (dgets(screen->fdin, buffer, IO_BUFFER_SIZE, 1) < 0)
+			{
+				say("IRCII exiting on EOF from stdin");
+				irc_exit(1, "EPIC - EOF from stdin");
+			}
+
+			if (strlen(buffer))
+				buffer[strlen(buffer) - 1] = 0;
+			parse_statement(buffer, 1, NULL);
+		}
+
+		/* Ordinary full screen input is handled one byte at a time */
+		else if ((n = dgets(screen->fdin, buffer, 
+					BIG_BUFFER_SIZE, -1)) > 0)
+		{
+			for (i = 0; i < n; i++)
+				translate_user_input(buffer[i]);
+		}
+
+		/* An EOF/error error on a wserv screen kills that screen */
+		else if (screen != main_screen)
+			kill_screen(screen);
+
+		/* An EOF/error on main screen kills the whole client. */
+		else
+			irc_exit(1, "Hey!  Where'd my controlling "
+					"terminal go?");
+
+		set_server_protocol_state(from_server, proto);
+	}
+
+	from_server = saved_from_server;
 } 
 
+/*
+ * This function accumulates bytes of user input encoded in 
+ * /SET INPUT_TRANSLATION and converts them to (UCS32) code points and
+ * calls edit_codepoint() once per code point.
+ */
+static	void translate_user_input (char byte)
+{
+static	char		workbuf[32];
+static	size_t		workbuf_idx = 0;
+	u_32int_t	codepoint;
+
+	codepoint = (u_32int_t)byte;
+	edit_codepoint(codepoint);
+}
+
 
 /* * * * * * * * * INPUT PROMPTS * * * * * * * * * * */
 /* 
@@ -3094,15 +3232,23 @@ void 	add_wait_prompt (const char *prompt, void (*func)(char *, char *), const c
 		change_input_prompt(1);
 }
 
+void	edit_codepoint (u_32int_t key)
+{
+	unsigned char	uckey;
+
+	uckey = (unsigned char)key;
+	edit_char(uckey);
+}
+
 /*
  * edit_char: handles each character for an input stream.  Not too difficult
  * to work out.
  */
 void    edit_char (unsigned char key)
 {
-        unsigned char          extended_key;
+        unsigned char   extended_key;
         WaitPrompt *    oldprompt;
-        unsigned char          dummy[2];
+        unsigned char   dummy[2];
         int             xxx_return = 0;         /* XXXX Need i say more? */
 
         if (dumb_mode)
