@@ -1,4 +1,4 @@
-/* $EPIC: server.c,v 1.231 2007/09/07 18:07:29 jnelson Exp $ */
+/* $EPIC: server.c,v 1.235 2008/02/19 13:13:12 crazyed Exp $ */
 /*
  * server.c:  Things dealing with that wacky program we call ircd.
  *
@@ -732,21 +732,29 @@ void 	display_server_list (void)
 		if (!(s = get_server(i)))
 			continue;
 
+		/*
+		 * XXX Ugh.  I should build this up bit by bit.
+		 */
 		if (!s->nickname)
-			say("\t%d) %s %d [%s] %s [%s]", i, s->info->host, s->info->port, 
+			say("\t%d) %s %d [%s] %s [%s] (vhost: %s)",
+				i, s->info->host, s->info->port, 
 				get_server_group(i), get_server_type(i),
-				server_states[get_server_status(i)]);
+				server_states[get_server_status(i)],
+				get_server_vhost(i));
 		else if (is_server_open(i))
-			say("\t%d) %s %d (%s) [%s] %s [%s]", 
+			say("\t%d) %s %d (%s) [%s] %s [%s] (vhost: %s)", 
 				i, s->info->host, s->info->port,
 				s->nickname, get_server_group(i),
 				get_server_type(i),
-				server_states[get_server_status(i)]);
+				server_states[get_server_status(i)],
+				get_server_vhost(i));
 		else
-			say("\t%d) %s %d (was %s) [%s] %s [%s]", i, s->info->host, 
+			say("\t%d) %s %d (was %s) [%s] %s [%s] (vhost: %s)", 
+				i, s->info->host, 
 				s->info->port, s->nickname, get_server_group(i),
 				get_server_type(i),
-				server_states[get_server_status(i)]);
+				server_states[get_server_status(i)],
+				get_server_vhost(i));
 	}
 }
 
@@ -1384,7 +1392,8 @@ return_from_ssl_detour:
 			    if (*end == '\r')
 				*end-- = '\0';
 
-			    l2 = message_from(NULL, LEVEL_OTHER);
+			    /* XXX We set message_from above. */
+			    /* l2 = message_from(NULL, LEVEL_OTHER); */
 			    if (x_debug & DEBUG_INBOUND)
 				yell("[%d] <- [%s]", 
 					s->des, buffer);
@@ -1394,7 +1403,7 @@ return_from_ssl_detour:
 			    parsing_server_index = i;
 			    parse_server(buffer, sizeof buffer);
 			    parsing_server_index = NOSERV;
-			    pop_message_from(l2);
+			    /* pop_message_from(l2); */
 			    break;
 		        }
 		    }
@@ -2260,6 +2269,7 @@ void  server_is_registered (int refnum, const char *itsname, const char *ourname
 					get_server_itsname(from_server));
 	window_check_channels();
 	set_server_status(refnum, SERVER_ACTIVE);
+	isonbase(from_server, NULL, NULL);
 }
 
 void	server_is_unregistered (int refnum)
@@ -2335,7 +2345,7 @@ BUILT_IN_COMMAND(disconnectcmd)
                 if (do_hook(DISCONNECT_LIST, "Disconnected by user request"))
 			say("You are not connected to a server, use /SERVER to connect.");
 
-	if (recon)
+	if (recon && 0 > connect_to_server(i))
 	{
 		set_server_status(i, SERVER_RECONNECT);
 		say("Reconnecting to server %s", get_server_itsname(i));
@@ -2882,7 +2892,7 @@ const char *	get_server_vhost (int servref )
 	if (!(s = get_server(servref)))
 		return "<none>";
 
-	if (s->info->vhost)
+	if (s->info->vhost && *s->info->vhost)
 		return s->info->vhost;
 	else
 		return "<none>";
